@@ -16,29 +16,29 @@
  *  Boston, MA 02110-1301, USA.
  **/
 
-#include "kservicefactory.h"
+#include "kservicefactory_p.h"
 #include "ksycoca.h"
 #include "ksycocatype.h"
 #include "ksycocadict_p.h"
 #include "kservice.h"
+#include "servicesdebug.h"
 #include <QDebug>
+#include <QDir>
+#include <QFile>
 
 extern int servicesDebugArea();
 
-Q_GLOBAL_STATIC(KSycocaFactorySingleton<KServiceFactory>, kServiceFactoryInstance)
-
-KServiceFactory::KServiceFactory()
-    : KSycocaFactory(KST_KServiceFactory),
-      m_nameDict(0),
-      m_relNameDict(0),
-      m_menuIdDict(0)
+KServiceFactory::KServiceFactory(KSycoca *db)
+    : KSycocaFactory(KST_KServiceFactory, db),
+      m_nameDict(nullptr),
+      m_relNameDict(nullptr),
+      m_menuIdDict(nullptr)
 {
-    kServiceFactoryInstance()->instanceCreated(this);
     m_offerListOffset = 0;
     m_nameDictOffset = 0;
     m_relNameDictOffset = 0;
     m_menuIdDictOffset = 0;
-    if (!KSycoca::self()->isBuilding()) {
+    if (!sycoca()->isBuilding()) {
         QDataStream *str = stream();
         Q_ASSERT(str);
         if (!str) {
@@ -68,19 +68,11 @@ KServiceFactory::KServiceFactory()
 
 KServiceFactory::~KServiceFactory()
 {
-    if (kServiceFactoryInstance()) {
-        kServiceFactoryInstance()->instanceDestroyed(this);
-    }
     delete m_nameDict;
     delete m_relNameDict;
     delete m_menuIdDict;
 }
 
-KServiceFactory *KServiceFactory::self()
-{
-    return kServiceFactoryInstance()->self();
-}
-
 KService::Ptr KServiceFactory::findServiceByName(const QString &_name)
 {
     if (!sycocaDict()) {
@@ -141,13 +133,13 @@ KService::Ptr KServiceFactory::findServiceByDesktopPath(const QString &_name)
 
     int offset = m_relNameDict->find_string(_name);
     if (!offset) {
-        //qDebug() << "findServiceByDesktopPath:" << _name << "not found";
+        //qCDebug(SERVICES) << "findServiceByDesktopPath:" << _name << "not found";
         return KService::Ptr(); // Not found
     }
 
     KService::Ptr newService(createEntry(offset));
     if (!newService) {
-        qDebug() << "createEntry failed!";
+        qCDebug(SERVICES) << "createEntry failed!";
     }
     // Check whether the dictionary was right
     // It's ok that it's wrong, for the case where we're looking up an unknown service,
@@ -183,23 +175,51 @@ KService::Ptr KServiceFactory::findServiceByMenuId(const QString &_menuId)
     return newService;
 }
 
+KService::Ptr KServiceFactory::findServiceByStorageId(const QString &_storageId)
+{
+    KService::Ptr service = findServiceByMenuId(_storageId);
+    if (service) {
+        return service;
+    }
+
+    service = findServiceByDesktopPath(_storageId);
+    if (service) {
+        return service;
+    }
+
+    if (!QDir::isRelativePath(_storageId) && QFile::exists(_storageId)) {
+        return KService::Ptr(new KService(_storageId));
+    }
+
+    QString tmp = _storageId;
+    tmp = tmp.mid(tmp.lastIndexOf(QLatin1Char('/')) + 1); // Strip dir
+
+    if (tmp.endsWith(QLatin1String(".desktop"))) {
+        tmp.truncate(tmp.length() - 8);
+    }
+
+    if (tmp.endsWith(QLatin1String(".kdelnk"))) {
+        tmp.truncate(tmp.length() - 7);
+    }
+
+    service = findServiceByDesktopName(tmp);
+
+    return service;
+}
+
 KService *KServiceFactory::createEntry(int offset) const
 {
-    KService *newEntry = 0L;
     KSycocaType type;
-    QDataStream *str = KSycoca::self()->findEntry(offset, type);
-    switch (type) {
-    case KST_KService:
-        newEntry = new KService(*str, offset);
-        break;
-    default:
+    QDataStream *str = sycoca()->findEntry(offset, type);
+    if (type != KST_KService) {
         qWarning() << "KServiceFactory: unexpected object entry in KSycoca database (type=" << int(type) << ")";
-        return 0;
+        return nullptr;
     }
+    KService *newEntry = new KService(*str, offset);
     if (!newEntry->isValid()) {
         qWarning() << "KServiceFactory: corrupt object in KSycoca database!";
         delete newEntry;
-        newEntry = 0;
+        newEntry = nullptr;
     }
     return newEntry;
 }
@@ -213,12 +233,19 @@ KService::List KServiceFactory::allServices()
     for (; it != end; ++it) {
         const KSycocaEntry::Ptr entry = *it;
         if (entry->isType(KST_KService)) {
-            result.append(QExplicitlySharedDataPointer<KService>(entry));
+            KService::Ptr service(static_cast<KService*>(entry.data()));
+            result.append(service);
         }
     }
     return result;
 }
 
+QStringList KServiceFactory::resourceDirs()
+{
+    return KSycocaFactory::allDirectories(QStringLiteral("kservices5"))
+           + KSycocaFactory::allDirectories(QStringLiteral("applications"));
+}
+
 QList<KServiceOffer> KServiceFactory::offers(int serviceTypeOffset, int serviceOffersOffset)
 {
     QList<KServiceOffer> list;
